from dataclasses import dataclass
from dacite import from_dict
from onlinesimru.api import Api
from typing import List, Dict, Optional, Union

@dataclass
class Payment:
    payment: float
    income: float
    spent: float
    now: float

@dataclass
class User:
    id: int
    name: str
    username: str
    email: Optional[str]
    apikey: Optional[str]
    api_access: Optional[Union[str,bool]]
    locale: str
    number_region: Optional[Union[str,int]]
    number_country: Optional[Union[str,int]]
    number_reject: Optional[List[Union[str,int]]]
    ugroup: int
    verify: int
    block: int
    payment: Payment


@dataclass
class Balance:
    balance: float
    zbalance: float
    income: float


class GetUser(Api):
    def balance(self):
        # return from_dict(data_class=Balance, data=self._get(f'/getBalance', {'income': True}))
        return self._get(f'/getBalance', {'income': True})

    def profile(self):
        return self._get(f'/getProfile')['profile']



class Error(Exception):
    """Base class for exceptions in this module."""
    pass


class NoNumberException(Error):
    def __init__(self, expression, message):
        self.expression = expression
        self.message = message


class RequestException(Error):
    def __init__(self, error: str, text: str = None):
        errors = {
            'ACCOUNT_BLOCKED': 'account blocked',
            'ERROR_WRONG_KEY': 'wrong apikey',
            'ERROR_NO_KEY': 'no apikey',
            'ERROR_NO_SERVICE': 'service not specified',
            'REQUEST_NOT_FOUND': 'API method not specified',
            'API_ACCESS_DISABLED': 'api disabled',
            'API_ACCESS_IP': 'access from this ip is disabled in the profile',
            'WARNING_NO_NUMS': 'no matching numbers',
            'TZ_INPOOL': 'waiting for a number to be dedicated to the operation',
            'TZ_NUM_WAIT': 'waiting for response',
            'TZ_NUM_ANSWER': 'response has arrived',
            'TZ_OVER_EMPTY': 'response did not arrive within the specified time',
            'TZ_OVER_OK': 'operation has been completed',
            'ERROR_NO_TZID': 'tzid is not specified',
            'ERROR_NO_OPERATIONS': 'no operations',
            'ACCOUNT_IDENTIFICATION_REQUIRED': 'You have to go through an identification process: to order a messenger - in any way, for forward - on the passport.',
            'EXCEEDED_CONCURRENT_OPERATIONS': 'maximum quantity of numbers booked concurrently is exceeded for your account',
            'NO_NUMBER': 'temporarily no numbers available for the selected service',
            'TIME_INTERVAL_ERROR': 'delayed SMS reception is not possible at this interval of time',
            'INTERVAL_CONCURRENT_REQUESTS_ERROR': 'maximum quantity of concurrent requests for number issue is exceeded, try again later',
            'TRY_AGAIN_LATER': 'temporarily unable to perform the request',
            'NO_FORWARD_FOR_DEFFER': 'forwarding can be activated only for online reception',
            'NO_NUMBER_FOR_FORWARD': 'there are no numbers for forwarding',
            'ERROR_LENGTH_NUMBER_FOR_FORWARD': 'wrong length of the number for forwarding',
            'DUPLICATE_OPERATION': 'adding operations with identical parameters',
            'ERROR_NO_NUMBER': 'number is not specified',
            'ERROR_PARAMS': 'one or both parameters are wrong',
            'LIFICYCLE_NUM_EXPIRED': 'the number has expired',
            'NEED_EXTENSION_NUMBER': 'you have to extend the number, see the Extension tab',
            'ERROR_NUMBERS_PARAMS': 'error in the number format',
            'ERROR_WRONG_TZID': 'error in the number format',
            'NO_COMPLETE_TZID': 'unable to complete the operation.',
            'NO_CONFIRM_FORWARD': 'unable to confirm forwarding',
            'ERROR_NO_SERVICE_REPEAT': 'no services for repeated reception',
            'SERVICE_TO_NUMBER_EMPTY': 'no numbers for repeated reception for this service',
        }

        if not text and errors.get(error):
            raise RequestException(error, errors.get(error))
        self.error = error
        self.text = text



from typing import List
from onlinesimru.api import Api


class GetForward(Api):
    def get(self, forward_numbers: List[int] = None, service: str = 'unlimited_sms', region: int = None, reject: List[int] = None):
        return self._get(f'/getForward', {'forward_numbers': forward_numbers, 'service': service, 'region': region, 'reject': reject})['tzid']

    def state(self, message_to_code: int = 1, orderby: str = 'ASC', msg_list: bool = True, clean: bool = True):
        return self._get(f'/getState', {'message_to_code': message_to_code, 'orderby': orderby, 'msg_list': msg_list, 'clean': clean, 'type': 'forward'})

    def stateOne(self, tzid: int, message_to_code: int = 1, orderby: str = 'ASC', msg_list: bool = True, clean: bool = True):
        return self._get(f'/getState', {'tzid': tzid, 'message_to_code': message_to_code, 'orderby': orderby, 'msg_list': msg_list, 'clean': clean, 'type': 'forward'})

    def close(self, tzid: int):
        return self._get(f'/setOperationOk', {'tzid': tzid})

    def repeat(self, service: int, number: int):
        return self._get(f'/getNumRepeat', {'service': service, 'number': number})['tzid']

    def tariffs(self):
        return self._get(f'/getNumbersStats', {'country': 'all'})

    def tariffsOne(self, country: int = 7):
        return self._get(f'/getNumbersStats', {'country': country})

    def service(self):
        return self._get(f'/getService', {})['service']

    def serviceNumber(self, service: str):
        return self._get(f'/getServiceNumber', {'service': service})['number']



from typing import List
from onlinesimru.api import Api
import time

class GetNumbers(Api):
    def price(self, service: str):
        return self._get(f'/getPrice', {'service': service})['price']

    def get(self, service: str, country: int = 7, reject: List[int] = None, extension: bool = False):
        return self._get(f'/getNum', {'service': service, 'country': country, 'reject': reject, 'extension': extension})['tzid']

    def state(self, message_to_code: int = 1, orderby: str = 'ASC', msg_list: bool = True, clean: bool = True, repeat: bool = False):
        type = 'index'
        if repeat:
            type = 'repeat'
        return self._get(f'/getState', {'message_to_code': message_to_code, 'orderby': orderby, 'msg_list': msg_list, 'clean': clean, 'type': type})

    def stateOne(self, tzid: int, message_to_code: int = 1, msg_list: bool = True, clean: bool = True, repeat: bool = False):
        type = 'index'
        if repeat:
            type = 'repeat'
        return self._get(f'/getState', {'tzid': tzid, 'message_to_code': message_to_code, 'msg_list': msg_list, 'clean': clean, 'type': type})[0]

    def next(self, tzid: int):
        return self._get(f'/setOperationRevise', {'tzid': tzid})

    def close(self, tzid: int):
        return self._get(f'/setOperationOk', {'tzid': tzid})

    def tariffs(self):
        return self._get(f'/getNumbersStats', {'country': 'all'})

    def tariffsOne(self, country: int = 7):
        return self._get(f'/getNumbersStats', {'country': country})

    def service(self):
        return self._get(f'getService', {})['service']

    def serviceNumber(self, service: str):
        return self._get(f'/getServiceNumber', {'service': service})['number']

    def wait_code(self, tzid: int, timeout=10, callback=None, not_end=False):
        __last_code: str = ''
        counter = 0
        while True:
            time.sleep(timeout)
            counter += 1
            if counter >= 10:
                raise ('Timeout error')
            response = self.stateOne(tzid, 1, False)
            if response['code'] and not not_end and response['code'] != __last_code:
                __last_code = response['code']
                self.close(tzid)
                break
            elif response['code'] and not_end and response['code'] != __last_code:
                __last_code = response['code']
                self.next(tzid)
                break
        if callback:
            callback(__last_code)

        return __last_code



# -*- coding: utf-8 -*-
from .GetForward import GetForward
from .GetFree import GetFree
from .GetNumbers import GetNumbers
from .GetProxy import GetProxy
from .GetRent import GetRent
from .GetUser import GetUser
from .Driver import Driver

__author__ = 's00d'
__version__ = '1.0'
__contact__ = 'https://github.com/s00d'



from os import path
from setuptools import setup

"""
@author: s00d
@contact: https://github.com/s00d
@license Apache License, Version 2.0, see LICENSE file
Copyright (C) 2018
"""


def long_description():
    """Build the description from README file """
    this_dir = path.abspath(path.dirname(__file__))
    with open(path.join(this_dir, 'README.md')) as f:
        return f.read()


def requirements():
    """Build the requirements list for this project"""
    requirements_list = list()
    with open('requirements.txt') as pc_requirements:
        for install in pc_requirements:
            requirements_list.append(install.strip())
    return requirements_list


setup(
    name='onlinesimru',
    version='1.0.4',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    description='Wrapper for automatic reception of SMS-messages by onlinesim.ru',
    author='s00d',
    license='Apache License, Version 2.0, see LICENSE file',
    keywords='sms, revice, onlinesim-ru, autoreg',
    author_email='suppport@onlinesim.ru',
    url='https://github.com/s00d/onlinesim-python-api',
    download_url='https://github.com/s00d/onlinesim-python-api/archive/master.zip',
    packages=['onlinesimru'],
    install_requires=requirements(),
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Environment :: Console',
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        'Programming Language :: Python :: Implementation :: PyPy',
    ]
)



from onlinesimru.api import Api


class GetProxy(Api):
    def get(self, cl:str = 'days', type:str = 'private', connect:str = 'https', count: int = 1, operator:str = None, country: int = 7, city: str = 'any', port_count: int = 1, session: bool = True):
        return self._get(f'/proxy/getProxy', {'class': cl, 'type': type, 'connect': connect, 'count': count, 'operator': operator, 'country': country, 'city': city, 'port_count': port_count, 'session': session})['item']

    def state(self, orderby: str = 'ASC'):
        return self._get(f'/proxy/getState', {'orderby': orderby})['list']

    def stateOne(self, tzid: int):
        return self._get(f'/proxy/getState', {'tzid': tzid})['list'][0]

    def changeIp(self, tzid: int):
        return self._get(f'/proxy/changeIp', {'tzid': tzid})

    def changeType(self, tzid: int):
        return self._get(f'/proxy/changeType', {'tzid': tzid})['connect_type']

    def setComment(self, tzid: int, comment: str = ''):
        return self._get(f'/proxy/setComment', {'tzid': tzid, 'comment': comment})



from onlinesimru import GetForward, GetFree, GetNumbers, GetProxy, GetRent, GetUser
from .api import Api


class Driver(Api):
    def forward(self):
        return GetForward(self.apikey, self.lang, self.lang)

    def free(self):
        return GetFree(self.apikey, self.lang, self.lang)

    def numbers(self):
        return GetNumbers(self.apikey, self.lang, self.lang)

    def proxy(self):
        return GetProxy(self.apikey, self.lang, self.lang)

    def rent(self):
        return GetRent(self.apikey, self.lang, self.lang)

    def user(self):
        return GetUser(self.apikey, self.lang, self.lang)